@@ -21,8 +21,9 @@
 #include "selectivesyncdialog.h"
 #include "accountstate.h"
 #include "creds/abstractcredentials.h"
+#include "wizard/owncloudwizard.h"
+#include "common/asserts.h"
 
-#include <QDebug>
 #include <QDesktopServices>
 #include <QDir>
 #include <QFileDialog>
@@ -34,11 +35,12 @@
 #include <QWizardPage>
 #include <QTreeWidget>
 #include <QVBoxLayout>
+#include <QEvent>
+#include <QCheckBox>
 
 #include <stdlib.h>
 
-namespace OCC
-{
+namespace OCC {
 
 QString FormatWarningsWizardPage::formatWarnings(const QStringList &warnings) const
 {
@@ -47,7 +49,7 @@ QString FormatWarningsWizardPage::formatWarnings(const QStringList &warnings) co
         ret = tr("<b>Warning:</b> %1").arg(warnings.first());
     } else if (warnings.count() > 1) {
         ret = tr("<b>Warning:</b>") + " <ul>";
-        Q_FOREACH(QString warning, warnings) {
+        Q_FOREACH (QString warning, warnings) {
             ret += QString::fromLatin1("<li>%1</li>").arg(warning);
         }
         ret += "</ul>";
@@ -56,48 +58,48 @@ QString FormatWarningsWizardPage::formatWarnings(const QStringList &warnings) co
     return ret;
 }
 
-FolderWizardLocalPath::FolderWizardLocalPath()
+FolderWizardLocalPath::FolderWizardLocalPath(const AccountPtr &account)
     : FormatWarningsWizardPage()
+    , _account(account)
 {
     _ui.setupUi(this);
     registerField(QLatin1String("sourceFolder*"), _ui.localFolderLineEdit);
-    connect(_ui.localFolderChooseBtn, SIGNAL(clicked()), this, SLOT(slotChooseLocalFolder()));
+    connect(_ui.localFolderChooseBtn, &QAbstractButton::clicked, this, &FolderWizardLocalPath::slotChooseLocalFolder);
     _ui.localFolderChooseBtn->setToolTip(tr("Click to select a local folder to sync."));
 
-    QString defaultPath = QString::fromLatin1( "%1/%2").arg( QDir::homePath() ).arg(Theme::instance()->appName() );
-    _ui.localFolderLineEdit->setText( QDir::toNativeSeparators( defaultPath ) );
+    QUrl serverUrl = _account->url();
+    serverUrl.setUserName(_account->credentials()->user());
+    QString defaultPath = QDir::homePath() + QLatin1Char('/') + Theme::instance()->appName();
+    defaultPath = FolderMan::instance()->findGoodPathForNewSyncFolder(defaultPath, serverUrl);
+    _ui.localFolderLineEdit->setText(QDir::toNativeSeparators(defaultPath));
     _ui.localFolderLineEdit->setToolTip(tr("Enter the path to the local folder."));
 
-    QString newAlias = Theme::instance()->appName();
-    int count = 0;
-    while (FolderMan::instance()->folder(newAlias)) {
-        // There is already a folder configured with this name and folder names need to be unique
-        newAlias = Theme::instance()->appName() + QString::number(++count);
-    }
-
     _ui.warnLabel->setTextFormat(Qt::RichText);
     _ui.warnLabel->hide();
 }
 
 FolderWizardLocalPath::~FolderWizardLocalPath()
 {
-
 }
 
 void FolderWizardLocalPath::initializePage()
 {
-  _ui.warnLabel->hide();
+    _ui.warnLabel->hide();
 }
 
 void FolderWizardLocalPath::cleanupPage()
 {
-  _ui.warnLabel->hide();
+    _ui.warnLabel->hide();
 }
 
 bool FolderWizardLocalPath::isComplete() const
 {
+    QUrl serverUrl = _account->url();
+    serverUrl.setUserName(_account->credentials()->user());
+
     QString errorStr = FolderMan::instance()->checkPathValidityForNewFolder(
-        QDir::fromNativeSeparators(_ui.localFolderLineEdit->text()));
+        QDir::fromNativeSeparators(_ui.localFolderLineEdit->text()), serverUrl);
+
 
     bool isOk = errorStr.isEmpty();
     QStringList warnStrings;
@@ -105,53 +107,46 @@ bool FolderWizardLocalPath::isComplete() const
         warnStrings << errorStr;
     }
 
-  _ui.warnLabel->setWordWrap(true);
-  if( isOk ) {
-    _ui.warnLabel->hide();
-    _ui.warnLabel->setText( QString::null );
-  } else {
-    _ui.warnLabel->show();
-    QString warnings = formatWarnings(warnStrings);
-    _ui.warnLabel->setText( warnings );
-  }
-  return isOk;
+    _ui.warnLabel->setWordWrap(true);
+    if (isOk) {
+        _ui.warnLabel->hide();
+        _ui.warnLabel->setText(QString());
+    } else {
+        _ui.warnLabel->show();
+        QString warnings = formatWarnings(warnStrings);
+        _ui.warnLabel->setText(warnings);
+    }
+    return isOk;
 }
 
 void FolderWizardLocalPath::slotChooseLocalFolder()
 {
-    QString sf = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
+    QString sf = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
     QDir d(sf);
 
     // open the first entry of the home dir. Otherwise the dir picker comes
     // up with the closed home dir icon, stupid Qt default...
     QStringList dirs = d.entryList(QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks,
-                                   QDir::DirsFirst|QDir::Name);
+        QDir::DirsFirst | QDir::Name);
 
-    if(dirs.count() > 0) sf += "/"+dirs.at(0); // Take the first dir in home dir.
+    if (dirs.count() > 0)
+        sf += "/" + dirs.at(0); // Take the first dir in home dir.
 
     QString dir = QFileDialog::getExistingDirectory(this,
-                                                    tr("Select the source folder"),
-                                                    sf);
+        tr("Select the source folder"),
+        sf);
     if (!dir.isEmpty()) {
         // set the last directory component name as alias
         _ui.localFolderLineEdit->setText(QDir::toNativeSeparators(dir));
-
-        QDir pickedDir(dir);
-        QString newAlias = pickedDir.dirName();
-        int count = 0;
-        while (FolderMan::instance()->folder(newAlias)) {
-            // There is already a folder configured with this name and folder names need to be unique
-            newAlias = pickedDir.dirName() + QString::number(++count);
-        }
     }
     emit completeChanged();
 }
 
 // =================================================================================
-FolderWizardRemotePath::FolderWizardRemotePath(AccountPtr account)
+FolderWizardRemotePath::FolderWizardRemotePath(const AccountPtr &account)
     : FormatWarningsWizardPage()
-    ,_warnWasVisible(false)
-    ,_account(account)
+    , _warnWasVisible(false)
+    , _account(account)
 
 {
     _ui.setupUi(this);
@@ -160,21 +155,19 @@ FolderWizardRemotePath::FolderWizardRemotePath(AccountPtr account)
     _ui.folderTreeWidget->setSortingEnabled(true);
     _ui.folderTreeWidget->sortByColumn(0, Qt::AscendingOrder);
 
-    connect(_ui.addFolderButton, SIGNAL(clicked()), SLOT(slotAddRemoteFolder()));
-    connect(_ui.refreshButton, SIGNAL(clicked()), SLOT(slotRefreshFolders()));
-    connect(_ui.folderTreeWidget, SIGNAL(itemExpanded(QTreeWidgetItem*)), SLOT(slotItemExpanded(QTreeWidgetItem*)));
-    connect(_ui.folderTreeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), SLOT(slotCurrentItemChanged(QTreeWidgetItem*)));
-    connect(_ui.folderEntry, SIGNAL(textEdited(QString)), SLOT(slotFolderEntryEdited(QString)));
+    connect(_ui.addFolderButton, &QAbstractButton::clicked, this, &FolderWizardRemotePath::slotAddRemoteFolder);
+    connect(_ui.refreshButton, &QAbstractButton::clicked, this, &FolderWizardRemotePath::slotRefreshFolders);
+    connect(_ui.folderTreeWidget, &QTreeWidget::itemExpanded, this, &FolderWizardRemotePath::slotItemExpanded);
+    connect(_ui.folderTreeWidget, &QTreeWidget::currentItemChanged, this, &FolderWizardRemotePath::slotCurrentItemChanged);
+    connect(_ui.folderEntry, &QLineEdit::textEdited, this, &FolderWizardRemotePath::slotFolderEntryEdited);
 
     _lscolTimer.setInterval(500);
     _lscolTimer.setSingleShot(true);
-    connect(&_lscolTimer, SIGNAL(timeout()), SLOT(slotLsColFolderEntry()));
+    connect(&_lscolTimer, &QTimer::timeout, this, &FolderWizardRemotePath::slotLsColFolderEntry);
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-    _ui.folderTreeWidget->header()->setSectionResizeMode(0,QHeaderView::ResizeToContents);
+    _ui.folderTreeWidget->header()->setSectionResizeMode(0, QHeaderView::ResizeToContents);
     // Make sure that there will be a scrollbar when the contents is too wide
     _ui.folderTreeWidget->header()->setStretchLastSection(false);
-#endif
 }
 
 void FolderWizardRemotePath::slotAddRemoteFolder()
@@ -190,14 +183,15 @@ void FolderWizardRemotePath::slotAddRemoteFolder()
 
     dlg->setWindowTitle(tr("Create Remote Folder"));
     dlg->setLabelText(tr("Enter the name of the new folder to be created below '%1':")
-                      .arg(parent));
+                          .arg(parent));
     dlg->open(this, SLOT(slotCreateRemoteFolder(QString)));
     dlg->setAttribute(Qt::WA_DeleteOnClose);
 }
 
 void FolderWizardRemotePath::slotCreateRemoteFolder(const QString &folder)
 {
-    if( folder.isEmpty() ) return;
+    if (folder.isEmpty())
+        return;
 
     QTreeWidgetItem *current = _ui.folderTreeWidget->currentItem();
     QString fullPath;
@@ -208,16 +202,16 @@ void FolderWizardRemotePath::slotCreateRemoteFolder(const QString &folder)
 
     MkColJob *job = new MkColJob(_account, fullPath, this);
     /* check the owncloud configuration file and query the ownCloud */
-    connect(job, SIGNAL(finished(QNetworkReply::NetworkError)),
-                 SLOT(slotCreateRemoteFolderFinished(QNetworkReply::NetworkError)));
-    connect(job, SIGNAL(networkError(QNetworkReply*)), SLOT(slotHandleMkdirNetworkError(QNetworkReply*)));
+    connect(job, static_cast<void (MkColJob::*)(QNetworkReply::NetworkError)>(&MkColJob::finished),
+        this, &FolderWizardRemotePath::slotCreateRemoteFolderFinished);
+    connect(job, &AbstractNetworkJob::networkError, this, &FolderWizardRemotePath::slotHandleMkdirNetworkError);
     job->start();
 }
 
 void FolderWizardRemotePath::slotCreateRemoteFolderFinished(QNetworkReply::NetworkError error)
 {
     if (error == QNetworkReply::NoError) {
-        qDebug() << "** webdav mkdir request finished";
+        qCDebug(lcWizard) << "webdav mkdir request finished";
         showWarn(tr("Folder was successfully created on %1.").arg(Theme::instance()->appNameGUI()));
         slotRefreshFolders();
         _ui.folderEntry->setText(static_cast<MkColJob *>(sender())->path());
@@ -227,22 +221,24 @@ void FolderWizardRemotePath::slotCreateRemoteFolderFinished(QNetworkReply::Netwo
 
 void FolderWizardRemotePath::slotHandleMkdirNetworkError(QNetworkReply *reply)
 {
-    qDebug() << "** webdav mkdir request failed:" << reply->error();
-    if( reply && !_account->credentials()->stillValid(reply) ) {
+    qCWarning(lcWizard) << "webdav mkdir request failed:" << reply->error();
+    if (!_account->credentials()->stillValid(reply)) {
         showWarn(tr("Authentication failed accessing %1").arg(Theme::instance()->appNameGUI()));
     } else {
         showWarn(tr("Failed to create the folder on %1. Please check manually.")
-                 .arg(Theme::instance()->appNameGUI()));
+                     .arg(Theme::instance()->appNameGUI()));
     }
 }
 
-void FolderWizardRemotePath::slotHandleLsColNetworkError(QNetworkReply *reply)
+void FolderWizardRemotePath::slotHandleLsColNetworkError(QNetworkReply * /*reply*/)
 {
+    auto job = qobject_cast<LsColJob *>(sender());
+    ASSERT(job);
     showWarn(tr("Failed to list a folder. Error: %1")
-             .arg(errorMessage(reply->errorString(), reply->readAll())));
+                 .arg(job->errorStringParsingBody()));
 }
 
-static QTreeWidgetItem* findFirstChild(QTreeWidgetItem *parent, const QString& text)
+static QTreeWidgetItem *findFirstChild(QTreeWidgetItem *parent, const QString &text)
 {
     for (int i = 0; i < parent->childCount(); ++i) {
         QTreeWidgetItem *child = parent->child(i);
@@ -294,7 +290,7 @@ bool FolderWizardRemotePath::selectByPath(QString path)
     QTreeWidgetItem *it = _ui.folderTreeWidget->topLevelItem(0);
     if (!path.isEmpty()) {
         const QStringList pathTrail = path.split(QLatin1Char('/'));
-        foreach (const QString& path, pathTrail) {
+        foreach (const QString &path, pathTrail) {
             if (!it) {
                 return false;
             }
@@ -323,11 +319,12 @@ void FolderWizardRemotePath::slotUpdateDirectories(const QStringList &list)
         root->setData(0, Qt::UserRole, "/");
     }
     QStringList sortedList = list;
-    sortedList.sort();
+    Utility::sortFilenames(sortedList);
     foreach (QString path, sortedList) {
         path.remove(webdavFolder);
         QStringList paths = path.split('/');
-        if (paths.last().isEmpty()) paths.removeLast();
+        if (paths.last().isEmpty())
+            paths.removeLast();
         recursiveInsert(root, paths, path);
     }
     root->setExpanded(true);
@@ -359,7 +356,7 @@ void FolderWizardRemotePath::slotCurrentItemChanged(QTreeWidgetItem *item)
     emit completeChanged();
 }
 
-void FolderWizardRemotePath::slotFolderEntryEdited(const QString& text)
+void FolderWizardRemotePath::slotFolderEntryEdited(const QString &text)
 {
     if (selectByPath(text)) {
         _lscolTimer.stop();
@@ -380,19 +377,19 @@ void FolderWizardRemotePath::slotLsColFolderEntry()
     // No error handling, no updating, we do this manually
     // because of extra logic in the typed-path case.
     disconnect(job, 0, this, 0);
-    connect(job, SIGNAL(finishedWithError(QNetworkReply*)),
-            SLOT(slotTypedPathError(QNetworkReply*)));
-    connect(job, SIGNAL(directoryListingSubfolders(QStringList)),
-            SLOT(slotTypedPathFound(QStringList)));
+    connect(job, &LsColJob::finishedWithError,
+        this, &FolderWizardRemotePath::slotTypedPathError);
+    connect(job, &LsColJob::directoryListingSubfolders,
+        this, &FolderWizardRemotePath::slotTypedPathFound);
 }
 
-void FolderWizardRemotePath::slotTypedPathFound(const QStringList& subpaths)
+void FolderWizardRemotePath::slotTypedPathFound(const QStringList &subpaths)
 {
     slotUpdateDirectories(subpaths);
     selectByPath(_ui.folderEntry->text());
 }
 
-void FolderWizardRemotePath::slotTypedPathError(QNetworkReply* reply)
+void FolderWizardRemotePath::slotTypedPathError(QNetworkReply *reply)
 {
     // Ignore 404s, otherwise users will get annoyed by error popups
     // when not typing fast enough. It's still clear that a given path
@@ -407,14 +404,14 @@ void FolderWizardRemotePath::slotTypedPathError(QNetworkReply* reply)
     slotHandleLsColNetworkError(reply);
 }
 
-LsColJob* FolderWizardRemotePath::runLsColJob(const QString& path)
+LsColJob *FolderWizardRemotePath::runLsColJob(const QString &path)
 {
     LsColJob *job = new LsColJob(_account, path, this);
     job->setProperties(QList<QByteArray>() << "resourcetype");
-    connect(job, SIGNAL(directoryListingSubfolders(QStringList)),
-            SLOT(slotUpdateDirectories(QStringList)));
-    connect(job, SIGNAL(finishedWithError(QNetworkReply*)),
-            SLOT(slotHandleLsColNetworkError(QNetworkReply*)));
+    connect(job, &LsColJob::directoryListingSubfolders,
+        this, &FolderWizardRemotePath::slotUpdateDirectories);
+    connect(job, &LsColJob::finishedWithError,
+        this, &FolderWizardRemotePath::slotHandleLsColNetworkError);
     job->start();
 
     return job;
@@ -438,8 +435,8 @@ bool FolderWizardRemotePath::isComplete() const
 
     Folder::Map map = FolderMan::instance()->map();
     Folder::Map::const_iterator i = map.constBegin();
-    for(i = map.constBegin();i != map.constEnd(); i++ ) {
-        Folder *f = static_cast<Folder*>(i.value());
+    for (i = map.constBegin(); i != map.constEnd(); i++) {
+        Folder *f = static_cast<Folder *>(i.value());
         if (f->accountState()->account() != _account) {
             continue;
         }
@@ -450,18 +447,12 @@ bool FolderWizardRemotePath::isComplete() const
         if (QDir::cleanPath(dir) == QDir::cleanPath(curDir)) {
             warnStrings.append(tr("This folder is already being synced."));
         } else if (dir.startsWith(curDir + QLatin1Char('/'))) {
-            warnStrings.append(tr("You are already syncing <i>%1</i>, which is a parent folder of <i>%2</i>.").arg(curDir).arg(dir));
-        }
-
-        if (curDir == QLatin1String("/")) {
-            warnStrings.append(tr("You are already syncing all your files. Syncing another folder is <b>not</b> supported. "
-                                  "If you want to sync multiple folders, please remove the currently configured "
-                                  "root folder sync."));
+            warnStrings.append(tr("You are already syncing <i>%1</i>, which is a parent folder of <i>%2</i>.").arg(Utility::escape(curDir), Utility::escape(dir)));
         }
     }
 
     showWarn(formatWarnings(warnStrings));
-    return warnStrings.isEmpty();
+    return true;
 }
 
 void FolderWizardRemotePath::cleanupPage()
@@ -475,25 +466,30 @@ void FolderWizardRemotePath::initializePage()
     slotRefreshFolders();
 }
 
-void FolderWizardRemotePath::showWarn( const QString& msg ) const
+void FolderWizardRemotePath::showWarn(const QString &msg) const
 {
-  if( msg.isEmpty() ) {
-    _ui.warnFrame->hide();
+    if (msg.isEmpty()) {
+        _ui.warnFrame->hide();
 
-  } else {
-    _ui.warnFrame->show();
-    _ui.warnLabel->setText( msg );
-  }
+    } else {
+        _ui.warnFrame->show();
+        _ui.warnLabel->setText(msg);
+    }
 }
 
 // ====================================================================================
 
-FolderWizardSelectiveSync::FolderWizardSelectiveSync(AccountPtr account)
+FolderWizardSelectiveSync::FolderWizardSelectiveSync(const AccountPtr &account)
 {
     QVBoxLayout *layout = new QVBoxLayout(this);
-    _treeView = new SelectiveSyncTreeView(account, this);
-    layout->addWidget(new QLabel(tr("Choose What to Sync: You can optionally deselect remote subfolders you do not wish to synchronize.")));
-    layout->addWidget(_treeView);
+    _selectiveSync = new SelectiveSyncWidget(account, this);
+    layout->addWidget(_selectiveSync);
+
+    if (Theme::instance()->showVirtualFilesOption()) {
+        _virtualFilesCheckBox = new QCheckBox(tr("Use virtual files instead of downloading content immediately (experimental)"));
+        connect(_virtualFilesCheckBox, &QCheckBox::clicked, this, &FolderWizardSelectiveSync::virtualFilesCheckboxClicked);
+        layout->addWidget(_virtualFilesCheckBox);
+    }
 }
 
 FolderWizardSelectiveSync::~FolderWizardSelectiveSync()
@@ -503,30 +499,49 @@ FolderWizardSelectiveSync::~FolderWizardSelectiveSync()
 
 void FolderWizardSelectiveSync::initializePage()
 {
-    QString alias        = wizard()->field(QLatin1String("alias")).toString();
-    QString targetPath   = wizard()->property("targetPath").toString();
+    QString targetPath = wizard()->property("targetPath").toString();
     if (targetPath.startsWith('/')) {
         targetPath = targetPath.mid(1);
     }
-    _treeView->setFolderInfo(targetPath, alias);
+    QString alias = QFileInfo(targetPath).fileName();
+    if (alias.isEmpty())
+        alias = Theme::instance()->appName();
+    QStringList initialBlacklist;
+    if (Theme::instance()->wizardSelectiveSyncDefaultNothing()) {
+        initialBlacklist = QStringList("/");
+    }
+    _selectiveSync->setFolderInfo(targetPath, alias, initialBlacklist);
     QWizardPage::initializePage();
 }
 
 bool FolderWizardSelectiveSync::validatePage()
 {
-    wizard()->setProperty("selectiveSyncBlackList", QVariant(_treeView->createBlackList()));
+    wizard()->setProperty("selectiveSyncBlackList", QVariant(_selectiveSync->createBlackList()));
+    wizard()->setProperty("useVirtualFiles", QVariant(_virtualFilesCheckBox && _virtualFilesCheckBox->isChecked()));
     return true;
 }
 
 void FolderWizardSelectiveSync::cleanupPage()
 {
-    QString alias        = wizard()->field(QLatin1String("alias")).toString();
-    QString targetPath   = wizard()->property("targetPath").toString();
-    _treeView->setFolderInfo(targetPath, alias);
+    QString targetPath = wizard()->property("targetPath").toString();
+    QString alias = QFileInfo(targetPath).fileName();
+    if (alias.isEmpty())
+        alias = Theme::instance()->appName();
+    _selectiveSync->setFolderInfo(targetPath, alias);
     QWizardPage::cleanupPage();
 }
 
-
+void FolderWizardSelectiveSync::virtualFilesCheckboxClicked()
+{
+    // The click has already had an effect on the box, so if it's
+    // checked it was newly activated.
+    if (_virtualFilesCheckBox->isChecked()) {
+        OwncloudWizard::askExperimentalVirtualFilesFeature([this](bool enable) {
+            if (!enable)
+                _virtualFilesCheckBox->setChecked(false);
+        });
+    }
+}
 
 
 // ====================================================================================
@@ -537,20 +552,22 @@ void FolderWizardSelectiveSync::cleanupPage()
  */
 
 FolderWizard::FolderWizard(AccountPtr account, QWidget *parent)
-    : QWizard(parent),
-    _folderWizardSourcePage(new FolderWizardLocalPath),
-    _folderWizardTargetPage(0),
-    _folderWizardSelectiveSyncPage(new FolderWizardSelectiveSync(account))
+    : QWizard(parent)
+    , _folderWizardSourcePage(new FolderWizardLocalPath(account))
+    , _folderWizardTargetPage(0)
+    , _folderWizardSelectiveSyncPage(new FolderWizardSelectiveSync(account))
 {
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
-    setPage(Page_Source, _folderWizardSourcePage );
+    setPage(Page_Source, _folderWizardSourcePage);
+    _folderWizardSourcePage->installEventFilter(this);
     if (!Theme::instance()->singleSyncFolder()) {
         _folderWizardTargetPage = new FolderWizardRemotePath(account);
-        setPage(Page_Target, _folderWizardTargetPage );
+        setPage(Page_Target, _folderWizardTargetPage);
+        _folderWizardTargetPage->installEventFilter(this);
     }
     setPage(Page_SelectiveSync, _folderWizardSelectiveSyncPage);
 
-    setWindowTitle( tr("Add Folder Sync Connection") );
+    setWindowTitle(tr("Add Folder Sync Connection"));
     setOptions(QWizard::CancelButtonOnLeft);
     setButtonText(QWizard::FinishButton, tr("Add Sync Connection"));
 }
@@ -559,6 +576,27 @@ FolderWizard::~FolderWizard()
 {
 }
 
+bool FolderWizard::eventFilter(QObject *watched, QEvent *event)
+{
+    if (event->type() == QEvent::LayoutRequest) {
+        // Workaround QTBUG-3396:  forces QWizardPrivate::updateLayout()
+        QTimer::singleShot(0, this, [this] { setTitleFormat(titleFormat()); });
+    }
+    return QWizard::eventFilter(watched, event);
+}
+
+void FolderWizard::resizeEvent(QResizeEvent *event)
+{
+    QWizard::resizeEvent(event);
+
+    // workaround for QTBUG-22819: when the error label word wrap, the minimum height is not adjusted
+    if (auto page = currentPage()) {
+        int hfw = page->heightForWidth(page->width());
+        if (page->height() < hfw) {
+            page->setMinimumSize(page->minimumSizeHint().width(), hfw);
+            setTitleFormat(titleFormat()); // And another workaround for QTBUG-3396
+        }
+    }
+}
 
 } // end namespace
-
